
package web.listeners;

import java.io.File;
import java.io.IOException;
import java.util.Properties;
import java.util.Timer;
import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import tasks.FileSweeper;
import util.FileUtil;

/**
  *@author University of Waikato - Te Whare Wānanga o Waikato
 * @version 1.0
 * @since   2014-11-20 
 */
public class MyServletContextListener implements ServletContextListener {

    private final String PROPERTIES_PATH = "../properties/web.properties";
    private final String WEB_DIRECTORY_NAME = "webtmpdir";

    /**
     * Creates a web temporary directory inside the temporary directory to use
     * as the temporary directory for the web application.
     * @param sce
     */
    public void contextInitialized(ServletContextEvent sce) {
        final File tmpdir = getTempDirectory();
        final File webtmpdir = new File(tmpdir, WEB_DIRECTORY_NAME);
        if (! webtmpdir.mkdir()) {
            System.err.println("Coult not create the servlet's temporary directory: " + webtmpdir.getAbsolutePath());
        }
        final ServletContext sc = sce.getServletContext();
        sc.setAttribute("tmpdir", webtmpdir.getAbsolutePath());

        //Fire off the file sweeper to run each hour.
        Timer timer = new Timer(true);
        timer.scheduleAtFixedRate(new FileSweeper(webtmpdir, 60), 60 * 60 * 1000, 60 * 60 * 1000);
    }

    /**
     * Deletes the web temporary directory inside the temporary directory
     * including all nested files.
     * @param sce
     */
    public void contextDestroyed(ServletContextEvent sce) {
        final ServletContext sc = sce.getServletContext();
        final File webtmpdir = new File((String)sc.getAttribute("tmpdir"));
        if (! FileUtil.deleteFile(webtmpdir)) {
            System.err.println("Could not delete the servlet's temporary directory: " + webtmpdir.getAbsolutePath());
        }
    }

    /**
     * Returns the temp directory property associated with the tmpdir key in
     * the web.properties file. If the property does not exist then the default
     * temp directory defined by System.getProperty("java.io.tmpdir") is returned.
     * @return the temporary directory.
     */
    private File getTempDirectory() {
        final Properties properties = new Properties();
        try {
            properties.load(getClass().getResourceAsStream(PROPERTIES_PATH));
        } catch (Exception e) {
	    System.err.println("getTempDirectory(): Did not find custom properties file: '"
			       + PROPERTIES_PATH + "'");
            System.err.println("Defaulting to Java property java.io.tmpdir="
				+ System.getProperty("java.io.tmpdir"));
	    
        }
        return new File(properties.getProperty("tmpdir", System.getProperty("java.io.tmpdir")));
    }
}
